<?php defined('SYSPATH') or die('No direct access allowed.');

/**
 * This class provides OSB invoice task capabilities.
 *
 * @package    Invoice
 * @category   Controllers
 * @author     Deon George
 * @copyright  (c) 2009-2013 Open Source Billing
 * @license    http://dev.osbill.net/license.html
 */
class Controller_Task_Invoice extends Controller_Task {
	public $auto_render = FALSE;

	public function action_send() {
		// Used to only process X invoices in a row.
		$max = ORM::factory('Invoice')->config('EMAIL_INV_MAX');

		$action = array();
		$iid = $this->request->param('id');
		$x = NULL;

		if (preg_match('/:/',$iid))
			list($iid,$x) = explode(':',$iid);

		// Get our list of invoices to send
		$i = $iid ? ORM::factory('Invoice')->where('id','=',$iid) : ORM::factory('Invoice')->list_tosend();

		$key = 'send';

		$max_count = 0;
		foreach ($i->find_all() as $io) {
			// If we have already sent a reminder or we dont email invoices we'll skip to the next one.
			if (($io->remind($key) AND (is_null($x) OR $x != 'again')) OR ($io->account->invoice_delivery != 1))
				continue;

			// If we have issued the max number of invoices this round, finish.
			if (++$max_count > $max)
				break;

			// Send our email
			$et = Email_Template::instance('task_invoice_'.$key);
			$token = ORM::factory('Module_Method_Token')
				->method(array('invoice','user_download'))
				->account($io->account)
				->expire(time()+86400*21)
				->uses(3)
				->generate();

			$et->to = array('account'=>array($io->account_id));
			$et->variables = array(
				'DUE'=>$io->due(TRUE),
				'DUE_DATE'=>$io->display('due_date'),
				'EMAIL'=>Company::instance()->email(),
				'FIRST_NAME'=>$io->account->first_name,
				'HTML_INVOICE'=>$io->html(),
				'INV_NUM'=>$io->refnum(),
				'INV_URL'=>URL::site(URL::link('user','invoice/view/'.$io->id),'http'),
				'INV_URL_DOWNLOAD'=>URL::site(URL::link('user',sprintf('invoice/download/%s?token=%s',$io->id,$token)),'http'),
				'SITE_NAME'=>Company::instance()->name(),
			);

			// @todo Record email log id if possible.
			if ($et->send()) {
				$io->print_status = 1;
				$io->set_remind($key,time(),($x=='again' ? TRUE : FALSE));
				array_push($action,(string)$io);
			}
		}

		$this->response->body(_('Invoices Sent: ').join('|',$action));
	}

	/** END **/

	public function action_audit_invoice_items() {
		$output = '';

		foreach (ORM::factory('Invoice_Item')->find_all() as $iio) {
			if ($iio->product_name AND $iio->product_id) {
				if (md5(strtoupper($iio->product_name)) == md5(strtoupper($iio->product->title()))) {
					$iio->product_name = NULL;
					$iio->save();
				} else {
					print_r(array("DIFF",'id'=>$iio->id,'pn'=>serialize($iio->product_name),'ppn'=>serialize($iio->product->title()),'pid'=>$iio->product_id,'test'=>strcasecmp($iio->product_name,$iio->product->title())));
				}
			}
		}

		$this->response->body($output);
	}
}
?>
